
all.ctries <- c("DE","FR","IT","SP","GR","PO","IR","BE","AU","FI","NL")
all.ctries.longnames <- c("Germany","France","Italy","Spain","Greece",
                          "Portugal","Ireland","Belgium","Austria","Finland","Netherlands")
GDP.euros.2018 <- c(3344,2353,1757,1202,185,202,324,451,386,232,774) # source, Eurostat https://ec.europa.eu/eurostat/databrowser/view/tec00001/default/table?lang=en

# Select countries used in the estimation:
ctry.names <- c("DE","FR","IT","SP","PO","NL","GR")
ctry.names <- c("DE","FR","IT","SP","GR")

n.e <- length(ctry.names) + 1 # "+1" is for riskfree entity
n.y <- n.e + 1
n.w <- n.y + n.e

# Select yields and CDS maturities used in the estimation:
maturities.in.years <- c(1,2,3,5,10)

max.CDS <- 20000 # remove CDS with non-reasonable values.


ALL.DATA <- read.csv2("~/Dropbox/Affine_Credit/Rcode_CDS/data/data4R.csv")
ALL.DATA$Date <- as.Date(ALL.DATA$Date,"%d.%m.%y")

DATA <- list(Date=ALL.DATA$Date)

# Get CDS DATA:

DATA$maturities.in.years <- maturities.in.years
CDS <- NULL
for(ctry in ctry.names){
  for(CDS.maturity in maturities.in.years){
    eval(parse(text = gsub(" ","",paste("CDS <- cbind(CDS,ALL.DATA$CDS",ctry,"EUR",toString(CDS.maturity),")",
                                        sep=""))))
  }
}
DATA$CDS <- CDS
# Remove non-reasonable CDS data:
DATA$CDS[DATA$CDS>max.CDS] <- NaN

# Remove illiquid data:
for(i in 1:dim(DATA$CDS)[2]){
  DATA$CDS[,i] <- remove.illiquid.data(DATA$CDS[,i],q=3)
}


# Get risk-free yields' DATA:

RF.yds <- NULL
for(maturity.in.years in maturities.in.years){
  if(maturity.in.years<1){
    eval(parse(text = gsub(" ","",paste("RF.yds <- cbind(RF.yds,ALL.DATA$OIS",toString(12*maturity.in.years),"M)",
                                        sep=""))))
  }else{
      eval(parse(text = gsub(" ","",paste("RF.yds <- cbind(RF.yds,ALL.DATA$ZC",toString(maturity.in.years),"Y)",
                                        sep=""))))
  }
}
DATA$RF.yds <- RF.yds

DATA$ctry.names <- ctry.names

# Observed delta's.
T <- length(DATA$Date)
DATA$obs.delta <- matrix(0,T,length(ctry.names))
if("GR" %in% ctry.names){
  where.is.Greece <- which(ctry.names=="GR")
  default.date <- as.Date("30.03.2012","%d.%m.%Y")
  where.is.default.date <- which(default.date==DATA$Date)
  
  DATA$obs.delta[where.is.default.date,where.is.Greece] <- - log(.24)
}

# Weights:
kappa <- NULL
for(i in 1:length(ctry.names)){
  where.is.ctry <- which(ctry.names[i]==all.ctries)
  kappa <- c(kappa,GDP.euros.2018[where.is.ctry])
}
kappa <- kappa/sum(kappa)
DATA$kappa <- kappa

# Prepare spread weights:
aux <- apply(DATA$CDS,2,function(x){sd(x,na.rm=TRUE)})
M <- diag(n.e-1) %x% matrix(1/length(DATA$maturities.in.years),1,length(DATA$maturities.in.years))
weights.spds <- M %*% aux

if("GR" %in% ctry.names){
  weights.spds[where.is.Greece] <- .5 * weights.spds[where.is.Greece]
}

mu.e <- rep(.6,n.e) # scale factor for delta_t's, calibrated on average Recovery Rates

