
# ===================================
# Initial model specification
# ===================================


# Number of entities (the first one is credit-risk free):
n.e <- 1 + length(ctry.names)

# Number of elements in y:
n.y <- 1 + (n.e-1) + 1 # 1 common factor + one variable per country (x) + 1 interest-rate factor.

# Dimension of w:
n.w <- n.y + n.e
# (w_t = [y_t',delta_t'])

EFFECTIVE.KAPPA <- kappa^.95

I <- .05 # indirect contagion parameter
S <- .2 * 4 # credit-event-pricing parameter
C <- c(0,.05*(1:(n.e-1))) # contagion parameter


theta.y <- .07
theta.x <- .01
theta.r <- 93

S <- .5 # credit-event-pricing parameter
theta.y <- .001
theta.x <- .001
theta.r <- 50

if(indic_no_z==1){
  beta.x.y <- 0 # influence of common factor on country-specific factors.
  nu_x <- 0.1
}else{
  beta.x.y <- .06 # influence of common factor on country-specific factors.
  nu_x <- 0
}


beta.lambda <- c(0,.000001*weights.spds) # effect of country specific factors on lambda

mu.r <- .000001
beta.r <- .98/mu.r
alpha.r <- .03/12 * (1 - beta.r * mu.r)/mu.r

# y_t is organized as follows:
# (common) y_t, x_t and r_t.

# Specifcation of y_t (common factor):
alpha.y <- c(0,rep(0,n.e-1),alpha.r)
beta.yy <- diag(c(.95,rep(.99,n.e-1),0))
beta.yy[2:n.e,1] <- beta.x.y
beta.yy[n.y,n.y] <- beta.r

beta.ye <- matrix(0,n.y,n.e) # this is beta_{e}^{(y)}, effect of delta on y
beta.ye[1,2:n.e] <- I * EFFECTIVE.KAPPA

mu.y <- c(rep(1,n.y-1),mu.r)
nu.y <- c(0.1,rep(nu_x,n.y-1))

# Specifcation of delta_t:
alpha.e <- rep(0,n.e)
beta.ey <- matrix(0,n.e,n.y) # this is beta_{y}^{(e)}, effect of y on delta
diag(beta.ey) <- beta.lambda

beta.ee <- matrix(C,ncol=1) %*% matrix(c(0,EFFECTIVE.KAPPA),nrow=1)

mu.e <- rep(.6,n.e)

# Short-term rate:
xi.0 <- 0
xi.1 <- rep(0,n.y+n.e)
xi.1[n.y] <- 1

a.0 <- rep(0,n.e)
a.1 <- rbind(
  matrix(0,n.y,n.e),
  diag(n.e)
)

theta.priceofrisk.y <- c(theta.y,theta.x*EFFECTIVE.KAPPA,theta.r)
theta.priceofrisk.delta <- S * c(0,EFFECTIVE.KAPPA)


# Build a list that contains the parameterization of the physical dynamics:
Model.ini <- list(
  n.e = n.e,
  n.y = n.y,
  xi.0=xi.0,xi.1=xi.1,
  a.0=a.0, a.1=a.1,
  alpha.y=alpha.y,alpha.e=alpha.e,
  mu.y=mu.y,mu.e=mu.e,
  nu.y=nu.y,
  beta.yy=beta.yy,beta.ey=beta.ey,beta.ee=beta.ee,beta.ye=beta.ye,
  n.e = n.e,
  n.y = n.y,
  theta.priceofrisk.y = theta.priceofrisk.y,
  theta.priceofrisk.delta = theta.priceofrisk.delta,
  CDS.std.err.measur = .1, # fraction of CDS std dev
  RF.std.err.measur = .2 # in percentage points
)

