
library(doParallel)


# Paths of models to be compared:

paths <- c(
  "results/param_baseline",
  "results/param_no_C",
  "results/param_no_S",
  "results/param_no_SC",
  "results/param_no_z",
  "results/param_no_Cz",
  "results/param_no_Sz",
  "results/param_no_SCz"
)

paths <- paste(paths,"_NEW",sep="")


# ================================================
number.of.cores <- 4
# ================================================

indic.cds <- 1

nb.loops <- 6
nb.sub.loops <- 1
mult.4.NlMd <- 100
mult.4.nlminb <- 1


cl <- makeCluster(number.of.cores)
registerDoParallel(cl)

save.image("toto.Rdata")
clusterEvalQ(cl,load("toto.Rdata"))

clusterEvalQ(cl,library(optimx))
clusterEvalQ(cl,library(MASS))


stime <- system.time({
  
  AUX <- foreach(specific.path=paths,.combine=rbind) %dopar% {
    
    load(file = paste(specific.path,".Rdat",sep=""))
    
    if(indic_shutdown_S==1){
      THETA[14] <- exp(-100)
    }
    if(indic_shutdown_I==1){
      THETA[5] <- exp(-100)
    }
    if(indic_shutdown_C==1){
      THETA[(18+1):(18+(n.e-1))] <- exp(-100)
    }
    if(indic_no_z==1){
      THETA[6] <- exp(-200) # beta.x.y, effect of common factor on country-specific factors.
      THETA[12] <- exp(-200) # theta.z
    }
    
    FILTER <- matrix(1,length(THETA))
    theta <- THETA.2.theta(THETA)
    Theta.ini <- theta[FILTER==1]
    Best.loglik <- loglik(Theta.ini,THETA,FILTER,DATA,kappa,weights.spds,
                          indic.cds = indic.cds,
                          indic.vec=0,
                          indic_max.avg.annual.SharpeRatio=indic_max.avg.annual.SharpeRatio)
    
    
    for(jj in 1:nb.loops){
      
      for(jjj in 0:1){
        
        FILTER <- matrix(0,length(THETA))
        
        if(jjj==0){
          FILTER[] <- 1
        }else{
          U <- runif(length(FILTER))
          FILTER <- (U<.3)
        }
        
        FILTER[1] <- 0 # Old beta.lambda
        FILTER[2] <- 0 # Old C
        
        if(indic_no_z==1){
          FILTER[3]  <- 0
          FILTER[4]  <- 0
          FILTER[5]  <- 0
          FILTER[6]  <- 0
        }
        
        if(indic_shutdown_S==1){
          FILTER[14] <- 0
        }
        if(indic_shutdown_I==1){
          FILTER[5] <- 0
        }
        if((indic_shutdown_C==1)&(indic_shutdown_I==1)){
          FILTER[(18+1):(18+(n.e-1))] <- 0
          FILTER[(18+(n.e-1)+1):(18+(n.e-1)+(n.e-1-1))] <- 0
        }
        if((indic_shutdown_C==1)&(indic_shutdown_I==0)){
          FILTER[(18+1):(18+(n.e-1))] <- 0
        }
        
        theta <- THETA.2.theta(THETA)
        Theta.ini <- theta[FILTER==1]
        
        for(i in 1:nb.sub.loops){
          
          for(method in c("Nelder-Mead","nlminb")){
            res.estim <- optimx(Theta.ini,loglik,
                                THETA = THETA,
                                FILTER =FILTER,
                                DATA = DATA,
                                kappa = kappa,
                                weights.spds = weights.spds,
                                indic.cds = indic.cds,
                                indic.vec=0,
                                indic_max.avg.annual.SharpeRatio = indic_max.avg.annual.SharpeRatio,
                                method = method,
                                control=list(trace=1,
                                             maxit = sum(FILTER)*ifelse(method=="Nelder-Mead",mult.4.NlMd,mult.4.nlminb),
                                             kkt = FALSE))
            
            if(
              (sum(is.na(as.matrix(res.estim)[1:length(Theta.ini)]))==0)&
              (res.estim$value<Best.loglik)&
              (res.estim$value>0)
            ){
              Best.loglik <- res.estim$value
              Theta.est <- c(as.matrix(res.estim)[1:length(Theta.ini)])
              Theta.ini <- Theta.est
            }
          }
          
          theta[FILTER==1] <- Theta.ini
          THETA            <- theta.2.THETA(theta)
          Model.est        <- THETA.2.MODEL(THETA,kappa,weights.spds)
          
          save(THETA,
               Model.est,
               DATA,
               kappa,weights.spds,
               indic.cds,indic_shutdown_S,indic_shutdown_I,indic_shutdown_C,indic_no_z,
               ctry.names,file = paste(specific.path,".TEMPO.Rdat",sep=""))
        }
      }
      
    }
    

    save(THETA,
         Model.est,
         DATA,
         kappa,weights.spds,
         indic.cds,indic_shutdown_S,indic_shutdown_I,indic_shutdown_C,indic_no_z,
         ctry.names,file = paste(specific.path,"_NEW.Rdat",sep=""))
  }
})
print(stime)

stopCluster(cl)
