####################################################
#
# Affine Modelling of Credit Risk, Pricing 
# of Credit Events and Contagion
#
# Monfort, Pegoraro, Renne & Roussellet, 2019
#
####################################################
##### Euro-are sovereign spreads  exercise #########
####################################################


# Clear environment and console
rm(list = ls(all = TRUE)) # clear environment 

# =============================================================
# Set path of working directory:
setwd("~/Dropbox/Affine_Credit/Rcode_CDS")
# =============================================================


# Select stored results:
path.stored.results <- "results/param_baseline.Rdat" # Model (1)
#path.stored.results <- "results/param_no_C.Rdat"     # Model (2)
#path.stored.results <- "results/param_no_S.Rdat"     # Model (3)
#path.stored.results <- "results/param_no_SC.Rdat"    # Model (4)
#path.stored.results <- "results/param_no_z.Rdat"     # Model (5)
#path.stored.results <- "results/param_no_Cz.Rdat"    # Model (6)
#path.stored.results <- "results/param_no_Sz.Rdat"    # Model (7)
#path.stored.results <- "results/param_no_SCz.Rdat"   # Model (8)


# Load libraries:
library(optimx)
library(MASS)
library(stringr)
library(stargazer)
library(lmtest)
library(sandwich)

# Modify plot default (margins) set up:
par(plt=c(.1,.9,.1,.9))

# Load procedures:
source("procedures/set.of.procedures.R")
source("procedures/linear_KF_withDelta.R")

# Load CDS, FX and yields data:
source("data/load_data.R")

max.value.4.S <- 10 # defines a maximum value for S parameter
# (usually not binding, this is just to stabilize the numerical optimization procedure)

fast.factor <- 3 # If 3 means that considers that the CDS spread is paid every 3 periods

# Maximum Sharpe ratio:
indic_max.avg.annual.SharpeRatio <- 1

# Load saved results:
load(file = path.stored.results)

# Prepare outputs:
source("prepare.outputs/make.charts.R")

