
# =========================================================
# Exercise on CDS quanto spreads
# =========================================================

# The present code has to be run after a preliminary run of "main.R" (same folder).

# ================================================
# Load estimated model and data:
# ================================================

load(file = "results/param_baseline.Rdat")


# check logl:
Model <- THETA.2.MODEL(THETA,kappa,weights.spds)
res.filter <- run.Filter(Model,DATA,
                         indic.cds=indic.cds, # 1 to use CDS formula (RMV otherwise)
                         indic.smoother = 1 # 1 to compute Risk Premiums
)
print(sum(res.filter$loglik.vector))


# If pprint==1, then figures will be saved:
pprint <- 1

# With depatreciation at default? (1 = Yes)
indic.no.yx.in.aws <- 1 # if 1, then z_t and x_t's do not affect s_t.
indic.allow.4.u.delta <- 1 # if 1, then delta_t affects s_t.

# Allow for price of risk associated with nu.
indic.with.theta <- 0


# ================================================
# Estimation (calibration) of s_t = log(FX_t)
# Delta(s_t) = Chi + v_{t} + u_{delta,s}'delta_t
# While u_{delta,s} is estimated by fitted quanto-CDSs,
# the dynamics of v_{t} is calibrated on moments.
# ================================================

a.0.s <- -.5 # this is Chi

TT <- length(ALL.DATA$EURUSD)
s <- log(ALL.DATA$EURUSD[2:TT])
plot(s,type="l")
delta.s <- -log(ALL.DATA$EURUSD[2:TT]) + log(ALL.DATA$EURUSD[1:(TT-1)])
plot(ALL.DATA$Date[2:TT],delta.s,type="l")
abline(v=default.date,col="red")
y.s <- delta.s - a.0.s
s.0 <- delta.s[2:length(y.s)]
s.1 <- delta.s[1:(length(y.s)-1)]
C <- var(cbind(s.0,s.1))
correl <- max(C[1,2]/C[1,1],0)
plot(y.s,type="l")

E <- - a.0.s
V <- var(y.s) * .000000001
rho.s <- correl

mu.s <- V/E
nu.s <- E/mu.s
beta.s <- 0


# Simulations aimed at checking realism of model:
nb.sim <- length(delta.s)
sim.y.s <- simul.model(0,beta.s,mu.s,nu.s,nb.sim,E)
par(mfrow=c(2,2))
plot(s,type="l",main="Observed s component")
sim.s <- s[1]+c(cumsum(sim.y.s + a.0.s))
plot(sim.s,type="l", main ="Simulated s component")
plot(ALL.DATA$EURUSD,type="l", main="Observed FX")
plot(exp(-sim.s),type="l",main="Simulated FX")



# ================================================
# Augmentation of Model with FX:
# ================================================

Extended.Model <- Model

old.n.y <- Model$n.y
n.e <- Model$n.e

alpha.s <- 0
theta.s <- 0
u.s     <- .01*kappa

Extended.Model$n.y <- Model$n.y + 1
Extended.Model$xi.1 <- c(Model$xi.1,0)
Extended.Model$a.1 <- rbind(rep(0,n.e),
                            Model$a.1)
Extended.Model$alpha.y <- c(Model$alpha.y,alpha.s)
Extended.Model$mu.y <- c(Extended.Model$mu.y,mu.s)
Extended.Model$nu.y <- c(Extended.Model$nu.y,nu.s)

AUX <- matrix(0,old.n.y+1,old.n.y+1)
AUX[1:old.n.y,1:old.n.y] <- Model$beta.yy
AUX[old.n.y+1,old.n.y+1] <- beta.s
Extended.Model$beta.yy <- AUX

Extended.Model$beta.ey <- cbind(Model$beta.ey,rep(0,n.e))
Extended.Model$beta.ye <- rbind(Model$beta.ye,rep(0,n.e))

Extended.Model$theta.priceofrisk.y <- c(Model$theta.priceofrisk.y,theta.s)

Extended.Model$a.w.s <- c(rep(0,old.n.y),
                          1, # loading on new factor
                          0, # delta of RF entity
                          u.s)
Extended.Model$a.0.s <- a.0.s



TTT <- dim(DATA$CDS)[1]

W <- res.filter$W.smoothed

# Augment matrices of factors with y_s:
W.FX <- rbind(res.filter$W.smoothed[1:old.n.y,],
              c(E,y.s), # add new factor
              res.filter$W.smoothed[(old.n.y+1):(old.n.y+n.e),])

# Compute values of Euro-denominated CDS:
u <- -10000
cds.pricing.Q <- pricing.CDS(make.Model.Q(Model),u,
                             12*max(DATA$maturities.in.years),W)
cds.pricing.P <- pricing.CDS(Model,u,
                             12*max(DATA$maturities.in.years),W)


# ===========================================
# Look for optimal u.s:
# ===========================================

# Build matrix of observed quanto CDSs:
quanto.CDS.obs <- array(NaN,c(n.e-1,TTT,length(DATA$maturities.in.years)))
quanto.CDS.obs.USD <- array(NaN,c(n.e-1,TTT,length(DATA$maturities.in.years)))
count <- 0
for(matur in 12*DATA$maturities.in.years){
  count <- count + 1
  for(j in 2:n.e){
    ctry <- ctry.names[j-1]
    eval(parse(text = gsub(" ","",
                           paste("spdUSD <- ALL.DATA$CDS",ctry,"USD",toString(round(matur/12,0)),sep="")
    )))
    eval(parse(text = gsub(" ","",
                           paste("spdEUR <- ALL.DATA$CDS",ctry,"EUR",toString(round(matur/12,0)),sep="")
    )))
    spdUSD <- remove.illiquid.data(spdUSD,q=3)
    spdEUR <- remove.illiquid.data(spdEUR,q=3)
    
    quanto.CDS.obs[j-1,,count] <- spdUSD - spdEUR
    quanto.CDS.obs.USD[j-1,,count] <- spdUSD
  }
}

# For Greece, compute the share of spreads accounted for by quanto spreads:
mean(quanto.CDS.obs[5,,]/quanto.CDS.obs.USD[5,,],na.rm=TRUE)

# Remove irrelevant data:
quanto.CDS.obs[abs(quanto.CDS.obs)==0] <- NaN



if(indic.no.yx.in.aws==1){
  if(indic.with.theta==1){
    PARAM <- c(log(u.s), # log of depreciations at default
               -5 # log of theta.s
    )
  }else{
    PARAM <- c(log(u.s)) # log of depreciations at default
  }
}else{
  if(indic.with.theta==1){
    PARAM <- c(rep(-10,Extended.Model$n.e),log(u.s),-5)
  }else{
    PARAM <- c(rep(-10,Extended.Model$n.e),log(u.s))
  }
  
}

PARAM.ini <- PARAM

# Launch numerical optimization:
nb.loops.quanto <- 2
max.iter.NldMd <- 400
max.iter.nlmind <- 10

for(iii in 1:nb.loops.quanto){
  for(method in c("Nelder-Mead","nlminb")){
    estim.param <- optimx(PARAM.ini,dev.quanto.4.optim,
                          Model.with.FX=Extended.Model,
                          W=W,delta.s=delta.s,DATA=DATA,
                          cds.pricing.Q=cds.pricing.Q,quanto.CDS.obs=quanto.CDS.obs,
                          method = method,
                          control=list(trace=1,
                                       maxit=ifelse(method=="Nelder-Mead",max.iter.NldMd,max.iter.nlmind),
                                       kkt = FALSE))
    PARAM.est <- c(as.matrix(estim.param)[1:length(PARAM)])
    PARAM.ini <- PARAM.est
  }
}




# ===========================================
# Prepare figure:
# ===========================================

# Build estimated FX model:
res.FX.estim <- make.quanto.Model(PARAM.est,Extended.Model,W,delta.s,DATA)

cds.pricing.Q.FX <- res.FX.estim$cds.pricing.Q.FX
cds.pricing.P.FX <- pricing.CDS(res.FX.estim$Model.with.FX,u,
                                max(12*DATA$maturities.in.years),res.FX.estim$W.FX)

# Express model-implied quanto-CDSs in basis points:
quanto.CDS.Q <- 120000*(cds.pricing.Q.FX$prices-cds.pricing.Q$prices)
quanto.CDS.P <- 120000*(cds.pricing.P.FX$prices-cds.pricing.P$prices)

quanto.Q1 <- quanto.CDS.Q

if(pprint==1){
  FILE = "/figures/Figure_quantoCDS.pdf"
  pdf(file=paste(getwd(),FILE,sep=""),pointsize=10,width=11, height=10)  
}

par(mfrow=c(length(12*DATA$maturities.in.years),n.e-1))
par(plt=c(.2,.95,.05,.75))

all.fit.table <- matrix(0,n.e-1,length(12*DATA$maturities.in.years))
all.RMSE.table <- matrix(0,n.e-1,length(12*DATA$maturities.in.years))
all.RMSquantoCDS.table <- matrix(0,n.e-1,length(12*DATA$maturities.in.years))

count <- 0
for(matur in 12*DATA$maturities.in.years){
  count <- count + 1
  for(j in 2:n.e){
    if(count==1){
      which.country <- which(ctry.names[j-1]==all.ctries)
      main.t <- all.ctries.longnames[which.country]
    }else{
      main.t <- ""
    }
    if(j==2){
      if(matur==12){
        ylab.t <- "1 year"
      }else{
        ylab.t=paste(toString(matur/12)," years",sep="")
      }
    }
    if((count==1)&(j==2)){
      factor.expand <- 2.3
    }else{
      factor.expand <- 1.3
    }
    
    ctry <- ctry.names[j-1]
    eval(parse(text = gsub(" ","",
                           paste("spdUSD <- ALL.DATA$CDS",ctry,"USD",toString(round(matur/12,0)),sep="")
    )))
    eval(parse(text = gsub(" ","",
                           paste("spdEUR <- ALL.DATA$CDS",ctry,"EUR",toString(round(matur/12,0)),sep="")
    )))
    
    spdUSD <- remove.illiquid.data(spdUSD,q=3)
    spdUSD[spdUSD>5000] <- NaN
    spdEUR <- remove.illiquid.data(spdEUR,q=3)
    spdEUR[spdEUR>5000] <- NaN
    
    quanto.CDS.obs <- spdUSD - spdEUR
    
    quanto.CDS.obs[abs(quanto.CDS.obs)==0] <- NaN
    
    modelled.quanto.CDS <- quanto.CDS.Q[j,,matur]
    modelled.quanto.CDS[is.na(quanto.CDS.obs)] <- NaN
    modelled.quanto.CDS.P <- quanto.CDS.P[j,,matur]
    modelled.quanto.CDS.P[is.na(quanto.CDS.obs)] <- NaN
    
    if(j>2){
      par(plt=c(.15,.95,.15,.8))
      ylab.t <- ""
    }else{
      par(plt=c(.25,.95,.15,.8))
    }
    plot(DATA$Date,quanto.CDS.obs,pch=3,
         main=main.t,cex.lab=1.7,cex.main=1.8,bty="n",
         ylim=c(0,
                factor.expand*max(c(modelled.quanto.CDS[!is.na(quanto.CDS.obs)],
                                    quanto.CDS.obs[!is.na(quanto.CDS.obs)]))),
         xlab="",ylab=ylab.t,las=1)

    if(ctry=="GR"){
      abline(v=DATA$Date[where.is.default.date],lty=2,lwd=1)
    }
    
    lines(DATA$Date,modelled.quanto.CDS,col="dark grey",lwd=2,lty=1)
    lines(DATA$Date,modelled.quanto.CDS.P,col="black",lwd=1,lty=1)
    errors <- quanto.CDS.obs - quanto.CDS.Q[j,,matur]
    all.fit.table[j-1,count] <- mean(errors^2,na.rm=TRUE)/var(quanto.CDS.obs[!is.na(errors)],na.rm=TRUE)
    all.RMSE.table[j-1,count] <- sqrt(mean(errors^2,na.rm=TRUE))
    all.RMSquantoCDS.table[j-1,count] <- sqrt(mean(quanto.CDS.obs^2,na.rm=TRUE))
    
    if((count==1)&(j==2)){
      legend("topright", # places a legend at the appropriate place
             c("Data","Model","Model (Physical)"),
             lty=c(NaN,1,1), # gives the legend appropriate symbols (lines)    
             pch=c(3,NaN,NaN),
             lwd=c(1,2,1), # line width
             col=c("black","dark grey","black"), # gives the legend lines the correct color and width
             bg="white",
             seg.len = 2,
             bty="n")
    }  
    
  }
  
}

if(pprint==1){
  dev.off()
}





# ===========================================
# Quantitative outputs:
# ===========================================


print(all.fit.table)

# Variance explained by model:
mean(1 - all.fit.table[1:4,])

# Effect of default on FX:
u.delta.i <- exp(PARAM.est[1:(n.e-1)])
(1 - Model$mu.e[1]*u.delta.i)^(-1) - 1

# Average RMSE per country:
apply(all.RMSE.table,1,mean)
apply(all.RMSquantoCDS.table,1,mean)
apply(all.RMSE.table^2/all.RMSquantoCDS.table^2,1,mean)


