
# Models to be compared:
paths <- c(
  "results/param_baseline", # Model (1)
  "results/param_no_C",     # Model (2)
  "results/param_no_S",     # Model (3)
  "results/param_no_SC",    # Model (4)
  "results/param_no_z",     # Model (5)
  "results/param_no_Cz",    # Model (6)
  "results/param_no_Sz",    # Model (7)
  "results/param_no_SCz"    # Model (8)
)

paths <- paste(paths,".Rdat",sep="")


T <- length(DATA$Date)

# Select forecats horizons:
all.horizons <- c(0,1,2,3)
all.name.Panels <- c("A","B","C","D","E","F","G")
maturities.in.years <- c(1,2,3,5,10)

keep.maturities <- NULL
for(j in 1:length(maturities.in.years)){
  aux <- which(maturities.in.years[j] == DATA$maturities.in.years)
  keep.maturities <- c(keep.maturities,aux)
}

# Define maturities (in months) for Table with ratios of Q/P.
all.horizons.4.QP.PD.ratios <- c(6,60,120)
all.horizons.4.QP.spreads.ratios <- c(6,60)



# ==========================================
# Prepare matrices of results:

all.fcst.CDS.t.t <- array(NaN,c(length(paths)+1,
                                n.e-1,
                                T,
                                length(maturities.in.years),
                                length(all.horizons)))
errors.all.fcst.CDS.t.t <- all.fcst.CDS.t.t
all.fcst.CDS.t.T        <- all.fcst.CDS.t.t
errors.all.fcst.CDS.t.T <- all.fcst.CDS.t.t

errors.all.fcst.chge.CDS.t.t <- all.fcst.CDS.t.t
errors.all.fcst.chge.CDS.t.T <- all.fcst.CDS.t.t

all.ratio.QP.PD <- array(NaN,
                         c(n.e-1,
                           length(all.horizons.4.QP.PD.ratios),
                           length(paths)))
all.ratio.QP.spreads <- array(NaN,
                              c(n.e-1,
                                length(all.horizons.4.QP.spreads.ratios),
                                length(paths)))

# ==========================================


all.lambda.P <- array(NaN,c(length(paths),T,n.e))
all.lambda.Q <- array(NaN,c(length(paths),T,n.e))

all.loglik.vector <- NULL


all.detect.nonnull.S <- NULL

for(i.model in 1:(length(paths)+1)){
  
  if(i.model <= length(paths)){
    load(file = paths[[i.model]])
  }
  
  all.detect.nonnull.S <- c(all.detect.nonnull.S,1-indic_shutdown_S)
  
  Model <- THETA.2.MODEL(THETA,kappa,weights.spds)
  res.filter <- run.Filter(Model,DATA,
                           indic.cds=indic.cds, # 1 to use CDS formula (RMV otherwise)
                           indic.smoother = 1 # 1 to compute Risk Premiums
  )
  print(sum(res.filter$loglik.vector))
  
  all.loglik.vector <- cbind(all.loglik.vector,res.filter$loglik.vector)
  
  if(i.model==1){
    all.Y.pred <- array(NaN,c(dim(res.filter$observable)[1],T,length(paths)))
    all.M.predicted <- array(NaN,c(dim(res.filter$observable)[1],dim(res.filter$observable)[1],T,length(paths)))
  }
  
  if(i.model <= length(paths)){
    all.Y.pred[,,i.model] <- res.filter$Y.predicted
    all.M.predicted[,,,i.model] <- res.filter$M.predicted
  }
  
  W.t.t <- res.filter$W.updated
  W.t.T <- res.filter$W.smoothed
  
  Model.Q <- make.Model.Q(Model)

  # Compute probabilities of default:
  PD.P <- compute.proba.def(Model,max.H,W.t.T)
  PD.Q <- compute.proba.def(Model.Q,max.H,W.t.T)
  mean.ratios <- apply((PD.Q/PD.P)[2:n.e,,],c(1,3),mean)
  if(i.model<=length(paths)){
    all.ratio.QP.PD[,,i.model] <- mean.ratios[,all.horizons.4.QP.PD.ratios]
  }
  
  lagged.delta <- W.t.T[(n.y+1):(n.y+n.e),]
  lagged.delta <- cbind(rep(NaN,n.e),lagged.delta[,1:(T-1)])
  
  if(i.model <= length(paths)){
    all.lambda.P[i.model,,] <- t(W.t.t[1:n.y,]) %*% t(Model$beta.ey) +
      t(lagged.delta) %*% t(Model$beta.ee)
    all.lambda.Q[i.model,,] <- t(W.t.t[1:n.y,]) %*% t(Model.Q$beta.ey) +
      t(lagged.delta) %*% t(Model.Q$beta.ee)
  }
  
  moments <- compute.moments(Model)
  
  M0 <- moments$M0
  M1 <- moments$M1
  
  Intercept <- 0*moments$M0
  Phi <- diag(n.y+n.e)
  
  
  count <- 0
  for(h in 0:max(all.horizons)){
    
    if(h==0){
      fcst.W.t.t <- Intercept %*% matrix(1,1,T) + Phi %*% W.t.t
      fcst.W.t.T <- Intercept %*% matrix(1,1,T) + Phi %*% W.t.T
      cds.pricing.t.t <- pricing.CDS(Model.Q,u=-10000,
                                     max(12 * maturities.in.years),fcst.W.t.t)
      cds.pricing.t.T <- pricing.CDS(Model.Q,u=-10000,
                                     max(12 * maturities.in.years),fcst.W.t.T)
      
      # Under P: ----------
      cds.pricing.t.t.Q <- cds.pricing.t.t
      cds.pricing.t.T.Q <- cds.pricing.t.T
      cds.pricing.t.t.P <- pricing.CDS(Model,u=-10000,
                                     max(12 * maturities.in.years),fcst.W.t.t)
      cds.pricing.t.T.P <- pricing.CDS(Model,u=-10000,
                                     max(12 * maturities.in.years),fcst.W.t.T)
      mean.ratios <- apply(((cds.pricing.t.T.Q$prices - cds.pricing.t.T.P$prices)/cds.pricing.t.T.Q$prices)[2:n.e,,],c(1,3),mean)
      if(i.model<=length(paths)){
        all.ratio.QP.spreads[,,i.model] <- mean.ratios[,all.horizons.4.QP.spreads.ratios]
      }
      # -------------------
      
      fcst.CDS.t.t <- array(120000*cds.pricing.t.t$prices[2:n.e,,12 * maturities.in.years],c(n.e-1,T,length(maturities.in.years)))
      fcst.CDS.t.T <- array(120000*cds.pricing.t.T$prices[2:n.e,,12 * maturities.in.years],c(n.e-1,T,length(maturities.in.years)))
      
      realized.CDS <- rbind(DATA$CDS[(h+1):T,rep(1,n.e-1)%x%keep.maturities+(0:(n.e-2))%x%rep(length(DATA$maturities.in.years),length(maturities.in.years))],
                            matrix(NaN,h,length(keep.maturities)*(n.e-1)))
      realized.CDS <- array(realized.CDS,c(T,length(maturities.in.years),n.e-1))
      realized.CDS <- aperm(realized.CDS, c(3,1,2))
      CDS.t.t.h0 <- fcst.CDS.t.t
      CDS.t.T.h0 <- fcst.CDS.t.T
      realized.CDS.h0 <- realized.CDS
    }
    
    if(h %in% all.horizons){
      
      count <- count + 1
      
      fcst.W.t.t <- Intercept %*% matrix(1,1,T) + Phi %*% W.t.t
      fcst.W.t.T <- Intercept %*% matrix(1,1,T) + Phi %*% W.t.T
      
      cds.pricing.t.t <- pricing.CDS(Model.Q,u=-10000,
                                     max(12 * maturities.in.years),fcst.W.t.t)
      cds.pricing.t.T <- pricing.CDS(Model.Q,u=-10000,
                                     max(12 * maturities.in.years),fcst.W.t.T)
      
      fcst.CDS.t.t <- array(120000*cds.pricing.t.t$prices[2:n.e,,12 * maturities.in.years],c(n.e-1,T,length(maturities.in.years)))
      fcst.CDS.t.T <- array(120000*cds.pricing.t.T$prices[2:n.e,,12 * maturities.in.years],c(n.e-1,T,length(maturities.in.years)))
      
      realized.CDS <- rbind(DATA$CDS[(h+1):T,rep(1,n.e-1)%x%keep.maturities+(0:(n.e-2))%x%rep(length(DATA$maturities.in.years),length(maturities.in.years))],
                            matrix(NaN,h,length(keep.maturities)*(n.e-1)))
      realized.CDS <- array(realized.CDS,c(T,length(maturities.in.years),n.e-1))
      realized.CDS <- aperm(realized.CDS, c(3,1,2))
      
      if(i.model <= length(paths)){
        all.fcst.CDS.t.t[i.model,,,,count] <- fcst.CDS.t.t
        all.fcst.CDS.t.T[i.model,,,,count] <- fcst.CDS.t.T
        
        errors.all.fcst.CDS.t.t[i.model,,,,count] <- fcst.CDS.t.t - realized.CDS
        errors.all.fcst.CDS.t.T[i.model,,,,count] <- fcst.CDS.t.T - realized.CDS
        
        errors.all.fcst.chge.CDS.t.t[i.model,,,,count] <- (fcst.CDS.t.t-CDS.t.t.h0) -
          (realized.CDS-realized.CDS.h0)
        errors.all.fcst.chge.CDS.t.T[i.model,,,,count] <- (fcst.CDS.t.T-CDS.t.T.h0) -
          (realized.CDS-realized.CDS.h0)
        
      }else{# Randow Walk case
        
        all.fcst.CDS.t.t[i.model,,,,count] <- realized.CDS.h0
        all.fcst.CDS.t.T[i.model,,,,count] <- realized.CDS.h0
        
        errors.all.fcst.CDS.t.t[i.model,,,,count] <- realized.CDS.h0 - realized.CDS
        errors.all.fcst.CDS.t.T[i.model,,,,count] <- realized.CDS.h0 - realized.CDS
        
        errors.all.fcst.chge.CDS.t.t[i.model,,,,count] <-  - (realized.CDS-realized.CDS.h0)
        errors.all.fcst.chge.CDS.t.T[i.model,,,,count] <-  - (realized.CDS-realized.CDS.h0)
      }
      
    }
    
    Intercept <- Intercept + Phi %*% M0
    Phi <- Phi %*% M1
    
  }
}


res.mse.fraction <- array(NaN,
                          c(length(paths)+1,
                            n.e-1,
                            length(all.horizons)))
res.mse.fraction.chge <- res.mse.fraction

# Compute ctry-specific spd std dev:
aux <- apply(DATA$CDS,2,function(x){sd(x,na.rm=TRUE)})
M <- diag(n.e-1) %x% matrix(1/length(DATA$maturities.in.years),1,length(DATA$maturities.in.years))
weights.std.dev <- M %*% aux

for(i in 1:length(all.horizons)){
  res.rmse <- apply(errors.all.fcst.CDS.t.t[,,,,i],c(1,2),function(X){sqrt(mean(X^2,na.rm = TRUE))})
  res.mse.fraction[,,i] <- res.rmse^2 / t(matrix(weights.std.dev^2,n.e-1,dim(res.rmse)[1]))
  
  print(res.rmse)
  print(res.mse.fraction[,,i])
  
  res.rmse.chge <- apply(errors.all.fcst.chge.CDS.t.t[,,,,i],c(1,2),function(X){sqrt(mean(X^2,na.rm = TRUE))})
  res.mse.fraction.chge[,,i] <- res.rmse.chge^2 / t(matrix(weights.std.dev^2,n.e-1,dim(res.rmse)[1]))
  
  print(res.rmse.chge)
  print(res.mse.fraction.chge[,,i])
}





# ===========================================
# ===========================================
# Prepare Latex tables
# ===========================================
# ===========================================



# ===========================================
# I - Model fit and forecasting performances

nb.decim <- 3

latex.table <- NULL

aux.string <- "Model"
for(i in 1:length(paths)){
  aux.string <- paste(aux.string,"&(",toString(i),")",sep="")
}


aux.string <- paste(aux.string,"&R.W.\\\\",sep="")
latex.table <- rbind(latex.table,aux.string,
                     "\\hline")

count.horiz <- 0
for(horiz in all.horizons){
  
  count.horiz <- count.horiz + 1
  
  if(all.horizons[count.horiz]==0){
    first.line <- paste("&\\multicolumn{",toString(length(paths)+1),
                        "}{l}{{\\bf Panel ",all.name.Panels[count.horiz]," -- Model fit (MSE)}}\\\\",
                        sep="")
  }else{
    first.line <- paste("&\\multicolumn{",toString(length(paths)+1),
                        "}{l}{{\\bf Panel ",all.name.Panels[count.horiz]," -- Forecast errors (horizon = ",toString(all.horizons[count.horiz]),
                        ifelse(all.horizons[count.horiz]==1," month)}"," months)}"),"}\\\\",
                        sep="")
  }
  
  latex.table <- rbind(latex.table,#"\\\\",
                       first.line,
                       "\\hline")
  
  # Detect best and second-best models.
  for(i.ctry in 1:(n.e-1)){
    
    AUX <- res.mse.fraction[,i.ctry,count.horiz]
    AUX[AUX==0] <- 1000000000
    where.is.best <- which(sort(AUX)[1]==AUX)
    where.is.2nd.best <- which(sort(AUX)[2]==AUX)
    where.is.3rd.best <- which(sort(AUX)[3]==AUX)
    
    
    ctry.name <- DATA$ctry.names[i.ctry]
    where.ctry <- which(all.ctries == ctry.name)
    ctry.long.name <- all.ctries.longnames[where.ctry]
    
    this.line <- ctry.long.name
    
    for(model in 1:dim(res.rmse)[1]){
      this.line <- paste(this.line,"&",
                         ifelse(res.mse.fraction[model,i.ctry,count.horiz]==0,"--",
                                round.fixed.length(res.mse.fraction[model,i.ctry,count.horiz],nb.decim)),
                         ifelse(model==where.is.best,"$^\\bullet$",""),
                         ifelse(model==where.is.2nd.best,"$^\\circ$",""),
                         ifelse(model==where.is.3rd.best,"",""),
                         sep="")
    }
    
    latex.table <- rbind(latex.table,this.line,"\\\\")
    
    latex.table <- rbind(latex.table,
                         "\\hline")
    
  }
  
}


latex.file <- paste("results/table_param_fcst.txt",sep="")
write(latex.table, file = latex.file)



# ===========================================
# II - Q/P ratios

nb.decim <- 3

latex.table <- NULL

aux.string <- "Model"
for(i in 1:length(paths)){
  aux.string <- paste(aux.string,"&(",toString(i),")",sep="")
}


aux.string <- paste(aux.string,"\\\\",sep="")
latex.table <- rbind(latex.table,aux.string,
                     "\\hline")

count.horiz <- 0
for(horiz in all.horizons.4.QP.spreads.ratios){
  
  count.horiz <- count.horiz + 1
  
  first.line <- paste("&\\multicolumn{",toString(length(paths)),
                      "}{l}{{\\bf Panel ",all.name.Panels[count.horiz]," -- Horizon = ",toString(all.horizons.4.QP.spreads.ratios[count.horiz]),
                      ifelse(all.horizons.4.QP.spreads.ratios[count.horiz]==1," month)}"," months}"),"}\\\\",
                      sep="")
  
  latex.table <- rbind(latex.table,#"\\\\",
                       first.line,
                       "\\hline")
  
  for(i.ctry in 1:(n.e-1)){
    ctry.name <- DATA$ctry.names[i.ctry]
    where.ctry <- which(all.ctries == ctry.name)
    ctry.long.name <- all.ctries.longnames[where.ctry]
    
    this.line <- ctry.long.name
    
    for(model in 1:dim(all.ratio.QP.spreads)[3]){
      this.line <- paste(this.line,"&",
                         ifelse(all.detect.nonnull.S[model]==1,"{\\bf ",""),
                         round.fixed.length(all.ratio.QP.spreads[i.ctry,count.horiz,model],nb.decim),
                         ifelse(all.detect.nonnull.S[model]==1,"}",""),
                         sep="")
      
    }
    this.line <- paste(this.line,"\\\\",sep="")
    
    latex.table <- rbind(latex.table,this.line)
    
    
  }
  latex.table <- rbind(latex.table,
                       "\\hline")
  
}


latex.file <- paste("results/table_QP_ratios.txt",sep="")
write(latex.table, file = latex.file)


