

pprint <- 1


# =================================================
# =================================================
# Load results:
# =================================================
# =================================================


# check logl:
Model <- THETA.2.MODEL(THETA,kappa,weights.spds)
system.time({
  res.filter <- run.Filter(Model,DATA,
                           indic.cds=indic.cds, # 1 to use CDS formula (RMV otherwise)
                           indic.smoother = 1, # 1 to compute Risk Premiums
                           indic_max.avg.annual.SharpeRatio = indic_max.avg.annual.SharpeRatio
  )
  print(sum(res.filter$loglik.vector))
})



n.e <- Model$n.e
n.y <- Model$n.y




# =================================================
# =================================================
# CDS fit
# =================================================
# =================================================

maturities.in.years <- c(1,5,10)

if(pprint==1){
  FILE = "/figures/Figure_CDS_fit.pdf"
  pdf(file=paste(getwd(),FILE,sep=""),pointsize=10,width=10, height=10)  
}

# Compute risk-neutral dynamics:
Model.Q <- make.Model.Q(Model)

W <- res.filter$W.smoothed

cds.pricing <- pricing.CDS(Model.Q,u=-10000,
                           max(12 * maturities.in.years),W)
cds.pricing.P <- pricing.CDS(Model,u=-10000,
                             max(12 * maturities.in.years),W)

dim(cds.pricing$prices)

par(mfrow=c(n.e-1,length(maturities.in.years)))
par(plt=c(.25,.95,.15,.8))

count <- 0
for(ctry in ctry.names){
  where.is.ctry <- which(ctry==all.ctries)
  
  count <- count + 1
  
  for(maturity.in.month in 12*maturities.in.years){
    
    main.title <- paste(all.ctries.longnames[where.is.ctry],", maturity: ",toString(maturity.in.month/12),
                        ifelse(maturity.in.month<=12," year"," years"),sep="")
    
    eval(parse(text = gsub(" ","",
                           paste("spd_data <- ALL.DATA$CDS",ctry,"EUR",
                                 toString(round(maturity.in.month/12,0)),sep="")
    )))
    spd_data[spd_data>max.CDS] <- NaN
    spd_data <- remove.illiquid.data(spd_data,q=3)
    
    modeled.price.Q <- 120000*cds.pricing$prices[count+1,,maturity.in.month]
    modeled.price.P <- 120000*cds.pricing.P$prices[count+1,,maturity.in.month]
    
    if((count==1)&(maturity.in.month==12*maturities.in.years[1])){
      y.lim <- c(0,1.5*max(c(spd_data,modeled.price.Q),na.rm=TRUE))
    }else{
      y.lim <- c(0,1*max(c(spd_data,modeled.price.Q),na.rm=TRUE))
    }
    if(ctry==ctry.names[1]){
      main.title <- paste("Maturity: ",toString(maturity.in.month/12),
                          ifelse(maturity.in.month<=12," year"," years"),sep="")
    }else{
      main.title <- ""
    }
    if(maturity.in.month==12*maturities.in.years[1]){
      ylab.t <- all.ctries.longnames[where.is.ctry]
    }else{
      ylab.t <- ""
    }
    plot(DATA$Date,modeled.price.Q,col="dark grey",lwd=2,type="l",
         ylim=y.lim,
         las=1,xlab="",ylab=ylab.t,bty="n",
         main=main.title,las=1,cex.main=1.7,cex.lab=2)
    points(DATA$Date,spd_data,pch=3,col="black")
    lines(DATA$Date,modeled.price.P,col="black",lwd=1,lty=1)
    
    if(ctry=="GR"){
      abline(v=DATA$Date[where.is.default.date],lty=2,lwd=1)
    }
    
    if((count==1)&(maturity.in.month==12*maturities.in.years[1])){
      legend("topright", # places a legend at the appropriate place 
             c("Data","Model-implied CDS","Model-implied CDS, without risk premiums"),
             lty=c(NaN,1,1), # gives the legend appropriate symbols (lines)       
             lwd=c(1,2,1), # line width
             col=c("black","dark grey","black"), # gives the legend lines the correct color and width
             pch = c(3,NaN,NaN),#symbols,
             bg="white",
             seg.len = 4,
             bty="n"
      )
    }  
    
  }
}

if(pprint==1){
  dev.off()
}







# =================================================
# =================================================
# Risk-free yields fit
# =================================================
# =================================================

if(pprint==1){
  FILE = "/figures/Figure_RFyields_fit.pdf"
  pdf(file=paste(getwd(),FILE,sep=""),pointsize=10,width=9, height=6)  
}

par(mfrow=c(2,3))

maturities.in.months <- 12*DATA$maturities.in.years
rmv.pricing.Q <- pricing.RMV(Model.Q,maturities.in.months,W)
rmv.pricing.P <- pricing.RMV(Model,maturities.in.months,W)

count <- 0
for(maturity.in.month in 12*DATA$maturities.in.years){
  count <- count + 1

  rf.yds.Q <- - 1200 / maturity.in.month * (
    rmv.pricing.Q$A.B$B[count,1] +
      t(W) %*% matrix(rmv.pricing.Q$A.B$A[,1,count],ncol=1))
  rf.yds.P <- - 1200 / maturity.in.month * (
    rmv.pricing.P$A.B$B[count,1] +
      t(W) %*% matrix(rmv.pricing.P$A.B$A[,1,count],ncol=1))
  
  main.title <- paste("Maturity: ",toString(maturity.in.month/12),
                      ifelse(maturity.in.month<=12," year"," years"),sep="")
  
  modeled.price.Q <- rf.yds.Q
  modeled.price.P <- rf.yds.P
  spd_data <- DATA$RF.yds[,count]
  
  if((count==1)&(maturity.in.month==12*maturities.in.years[1])){
    y.lim <- c(0,1.5*max(c(spd_data,modeled.price.Q),na.rm=TRUE))
  }else{
    y.lim <- c(0,1.1*max(c(spd_data,modeled.price.Q),na.rm=TRUE))
  }
  plot(DATA$Date,modeled.price.Q,col="dark grey",lwd=2,type="l",
       ylim=y.lim,
       las=1,xlab="",ylab="",bty="n",
       main=main.title)
  points(DATA$Date,spd_data,pch=3,col="black")
  lines(DATA$Date,modeled.price.P,col="black",lwd=1,lty=1)
  
  if((count==1)&(maturity.in.month==12*maturities.in.years[1])){
    legend("topright", # places a legend at the appropriate place c("Health","Defense"), # puts text in the legend 
           c("Data","Model-implied yield","Model-implied yield, without risk premiums"),
           lty=c(NaN,1,1), # gives the legend appropriate symbols (lines)       
           lwd=c(1,2,1), # line width
           col=c("black","dark grey","black"), # gives the legend lines the correct color and width
           pch = c(3,NaN,NaN),#symbols,
           bg="white",
           seg.len = 4,
           bty="n"
    )
  }  
  
  
}

if(pprint==1){
  dev.off()
}




# ==================================================================
# ==================================================================
# Compute Term structures of CDS yield curves for specific dates.
# ==================================================================
# ==================================================================

dates <- c(45,128)

nb.mat <- length(DATA$maturities.in.years)

Model.no.S <- Model
Model.no.S$theta.priceofrisk.delta[] <- 0

cds.pricing.Q <- pricing.CDS(make.Model.Q(Model),u=-10000,
                             max(12 * DATA$maturities.in.years),W)
cds.pricing.P <- pricing.CDS(Model,u=-10000,
                             max(12 * DATA$maturities.in.years),W)

# Load models obtained with u.delta = 0:
cds.pricing.Q0 <- pricing.CDS(make.Model.Q(Model.no.S),u=-10000,
                              max(12 * DATA$maturities.in.years),W)
cds.pricing.P0 <- pricing.CDS(Model.no.S,u=-10000,
                              max(12 * DATA$maturities.in.years),W)


if(pprint==1){
  FILE = "/figures/Figure_TSPDs.pdf"
  pdf(file=paste(getwd(),FILE,sep=""),pointsize=10,width=8, height=5)  
}

par(mfrow=c(length(dates),length(ctry.names)))
par(plt=c(.3,.95,.2,.85))

for(i in 1:length(dates)){
  for(j in 2:n.e){
    
    if(i==1){
      which.country <- which(ctry.names[j-1]==all.ctries)
      main.title <- all.ctries.longnames[which.country]
    }else{
      main.title <- ""
    }
    if(j==2){
      ylab.t <- DATA$Date[dates[i]]
    }else{
      ylab.t <- ""
    }
    
    plot(120000*cds.pricing.Q$prices[j,dates[i],],type="l",lwd=2,
         main=main.title,
         ylim=c(0,1.7*max(120000*cds.pricing.Q$prices[j,dates[i],])),
         xlab="",ylab=ylab.t,bty="n",
         las=1,
         cex.main=1.5,cex.lab=1.5)
    
    text(60,0.01,"Maturity (in months)")
    lines(120000*cds.pricing.P$prices[j,dates[i],],col="dark grey",lwd=2)
    lines(120000*cds.pricing.P0$prices[j,dates[i],],col="dark grey",lwd=2,lty=3)
    lines(120000*cds.pricing.Q0$prices[j,dates[i],],col="black",lwd=2,lty=3)
    points(maturities.in.months,DATA$CDS[dates[i],
                                         nb.mat*(j-1-1)+(1:nb.mat)],pch=19,cex=1.5,col="dark grey")
    
    if((i==1)&(j==2)){
      legend("topleft", # places a legend at the appropriate place c("Health","Defense"), # puts text in the legend 
             c("Data","Model","Model (physical)"),
             lty=c(NaN,1,1), # gives the legend appropriate symbols (lines)       
             lwd=c(NaN,2,2), # line width
             col=c("dark grey","black","dark grey"), # gives the legend lines the correct color and width
             pch = c(19,NaN,NaN),#symbols,
             bg="white",
             seg.len = 2,
             bty="n"
      )
    }
    
  }
}

if(pprint==1){
  dev.off()
}





# ======================================================
# ======================================================
# Compare P and Q probabilities of default.
# ======================================================
# ======================================================

if(pprint==1){
  FILE = "/figures/Figure_PDs.pdf"
  pdf(file=paste(getwd(),FILE,sep=""),pointsize=10,width=10, height=6)  
}

max.H <- 120
PD.P <- compute.proba.def(Model,max.H,W)
PD.Q <- compute.proba.def(make.Model.Q(Model),max.H,W)

maturities.4.PD <- c(12,60)

par(mfrow=c(length(maturities.4.PD),n.e-1))

for(j in 1:length(maturities.4.PD)){
  for(i in 1:(n.e-1)){
    
    lty.t = 1
    
    where.is.ctry <- which(ctry.names[i]==all.ctries)
    main.t <- paste(all.ctries.longnames[where.is.ctry],", horizon = ",toString(maturities.4.PD[j])," months",sep="")
    
    
    main.t <- ifelse(j==1,all.ctries.longnames[where.is.ctry],"")
    ylab.t <- ifelse(i==1,
                     paste("Horizon = ",toString(maturities.4.PD[j])," months",sep=""),"")
    if(i==1){
      par(plt=c(.25,.95,.1,.9))
    }else{
      par(plt=c(.15,.95,.1,.9))
    }
    
    plot(DATA$Date,100*PD.Q[i+1,,maturities.4.PD[j]],type="l",
         main=main.t,
         lty=1,col="dark grey",lwd=2,
         ylim=c(0,min(1.2*max(100*PD.Q[i+1,,maturities.4.PD[j]]),100)),
         xlab="",ylab=ylab.t,las=1,bty="n",
         cex.main=1.5,cex.lab=1.5)
    lines(DATA$Date,100*PD.P[i+1,,maturities.4.PD[j]],lty=lty.t,col="black",lwd=2)
    abline(h=0)
    if(ctry.names[i]=="GR"){
      abline(v=DATA$Date[where.is.default.date],lty=2)
    }
    if((i==1)&(j==1)){
      legend("topright", # places a legend at the appropriate place c("Health","Defense"), # puts text in the legend 
             c("Physical","Risk-neutral"),
             lty=c(1,1), # gives the legend appropriate symbols (lines)       
             lwd=c(2,2), # line width
             col=c("black","dark grey"), # gives the legend lines the correct color and width
             bg="white",
             seg.len = 4,title="Probability of default:",
             bty="n"
      )
    }
  }
}

if(pprint==1){
  dev.off()
}






# ======================================================
# ======================================================
# Plot factors
# ======================================================
# ======================================================

if(pprint==1){
  FILE = "/figures/Figure_Factors.pdf"
  pdf(file=paste(getwd(),FILE,sep=""),pointsize=10,width=7, height=7)  
}

par(mfrow=c(round((n.e+2)/2),2))

par(plt=c(.15,.95,.2,.8))

# prepare time series of standard deviations:
W.stdv <- NULL
for(i in 1:dim(W)[1]){
  W.stdv <- cbind(W.stdv,sqrt(res.filter$P.smoothed[i,i,]))
}

multipl.factor <- 1.96

T <- length(DATA$Date)

# The short-term rate:
main.t <- expression(paste("Short-term rate ",r[t], " (in percent)",sep=""))
indic <- n.e + 1
plot(DATA$Date,1200*W[indic,],type="l",las=1,
     main=expression(paste("Short-term rate ",r[t], " (in percent)"),sep=""),
     xlab="",ylab="",bty="n")
polygon(x=c(DATA$Date[1:T],DATA$Date[T:1]),
        y=1200*c(W[indic,1:T] - multipl.factor*W.stdv[1:T,indic],
                 W[indic,T:1] + multipl.factor*W.stdv[T:1,indic]),
        border = NA,col = "grey")
lines(DATA$Date,1200*W[indic,],lwd=2)

# Common factor:
main.t <- expression(paste("Common factor ",z[t],sep=""))
indic <- 1
plot(DATA$Date,W[indic,],type="l",las=1,
     main=main.t,
     xlab="",ylab="",bty="n")
polygon(x=c(DATA$Date[1:T],DATA$Date[T:1]),
        y=c(W[indic,1:T] - multipl.factor*W.stdv[1:T,indic],
            W[indic,T:1] + multipl.factor*W.stdv[T:1,indic]),
        border = NA,col = "grey")
lines(DATA$Date,W[indic,],lwd=2)

for(i in 1:(n.e-1)){
  
  indic <- 1 + i
  
  ctry <- ctry.names[i]
  where.is.ctry <- which(ctry==all.ctries)
  long.ctry.name <- all.ctries.longnames[where.is.ctry]
  
  if(ctry.names[i]=="DE"){
    main.t <- expression(paste("German-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="FR"){
    main.t <- expression(paste("France-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="IT"){
    main.t <- expression(paste("Italy-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="SP"){
    main.t <- expression(paste("Spain-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="GR"){
    main.t <- expression(paste("Greece-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="PO"){
    main.t <- expression(paste("Portugal-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="IR"){
    main.t <- expression(paste("Ireland-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="BE"){
    main.t <- expression(paste("Belgium-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="AU"){
    main.t <- expression(paste("Austria-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="FI"){
    main.t <- expression(paste("Finland-specific factor (",x[i*','*t],")",sep=""))
  }
  if(ctry.names[i]=="NL"){
    main.t <- expression(paste("Netherlands-specific factor (",x[i*','*t],")",sep=""))
  }
  
  plot(DATA$Date,W[indic,],type="l",las=1,
       main=main.t,
       xlab="",ylab="",bty="n")
  polygon(x=c(DATA$Date[1:T],DATA$Date[T:1]),
          y=c(W[indic,1:T] - multipl.factor*W.stdv[1:T,indic],
              W[indic,T:1] + multipl.factor*W.stdv[T:1,indic]),
          border = NA,col = "grey")
  lines(DATA$Date,W[indic,],lwd=2)
  if(ctry=="GR"){
    abline(v=DATA$Date[where.is.default.date],lty=2,lwd=1)
  }
}


if(pprint==1){
  dev.off()
}




