
# Prepare regression table (frailty factor / macro-finance variables)

AUX <- DATA$CDS
sd.col.AUX <- apply(AUX,2,function(x){sd(x,na.rm=TRUE)})
mean.col.AUX <- apply(AUX,2,function(x){mean(x,na.rm=TRUE)})
AUX <- (AUX - t(matrix(mean.col.AUX,length(sd.col.AUX),dim(AUX)[1]))) / t(matrix(sd.col.AUX,length(sd.col.AUX),dim(AUX)[1]))
M <- compute.covar.with.na(AUX)
eig.M <- eigen(M)
PC1 <- matrix(NaN,dim(AUX)[1],1)
for(t in 1:dim(AUX)[1]){
  aux <- AUX[t,]
  indic.not.na <- which(!is.na(aux))
  weights <- eig.M$vectors[indic.not.na,1]/sum(eig.M$vectors[indic.not.na,1])
  PC1[t] <- sum(aux[indic.not.na] * weights)
}
ALL.DATA$PC1 <- PC1


Y.variable <- res.filter$W.smoothed[1,]

all.cor <- NULL
for(i in 1:dim(DATA$CDS)[2]){
  X <- DATA$CDS[,i]
  where.is.not.na <- which(!is.na(X))
  a <- cor(X[where.is.not.na],Y.variable[where.is.not.na])
  all.cor <- c(all.cor,a)
}
mean(all.cor)


select.covariates <- c("ECONOMIC.POLICY.UNCERTAINTY",
                       "VSTOXX",
                       "SOVEREIGN.STRESS.EQUAL.WEIGHTS",
                       "UNEMPL",
                       "ESI",
                       "Google.sovereign.debt.crisis")

for(i in 1:length(select.covariates)){
  eval(parse(text = gsub(" ","",paste("eq",toString(i)," <- lm(Y.variable~ALL.DATA$",select.covariates[i],")",
                                      sep=""))))
  
  eval(parse(text = gsub(" ","",paste("vCov",toString(i)," <- NeweyWest(eq",toString(i),")",sep=""))))
  eval(parse(text = gsub(" ","",paste("coeftest",toString(i)," <- coeftest(eq",toString(i),",vCov",toString(i),")",sep=""))))
}

eqALL <- lm(Y.variable~
              ALL.DATA$ECONOMIC.POLICY.UNCERTAINTY+
              ALL.DATA$Google.sovereign.debt.crisis
)
vCovALL <- NeweyWest(eqALL,lag=2)
coeftestALL <- coeftest(eqALL,vCovALL)

table.regression <- stargazer(eq1,eq2,eq3,eq4,eq5,eq6,
                              omit.stat=c("f", "ser"),
                              intercept.bottom = FALSE,
                              dep.var.labels = "Common factor $z_t$",
                              covariate.labels = c("Intercept",
                                                   "EPU",
                                                   "VSTOXX",
                                                   "CISS",
                                                   "Unempl.",
                                                   "ESI",
                                                   "Google"),
                              se = list(coeftest1[,2],coeftest2[,2],coeftest3[,2],
                                        coeftest4[,2],coeftest5[,2],coeftest6[,2]),
                              p = list(coeftest1[,4],coeftest2[,4],coeftest3[,4],
                                       coeftest4[,4],coeftest5[,4],coeftest6[,4]),
                              notes.align = "l",
                              label="Tabregressions",
                              title="Regression of common factor ($z_t$) on economic and financial stress indicators",
                              notes="\\parbox{.6\\linewidth}{\\vspace{2pt}%stars. \\\\
                              EPU is the European Economic Policy Uncertainty indicator;
                              CISS is a Composite Indicator of Systemic Stress; ESI is the European Sentiment Index; `Google' is the number of Google searches for `sovereign debt crisis'. Standard deviations are corrected for heteroskedasticity and autocorrelation of residuals \\citep{Newey_West_1987}}")


writeLines(table.regression, "results/tabregression.txt")







