#==========================================================================
# This program builds the table of parameter estimates
# associated with the different models
#==========================================================================


# Paths of models to be compared:
paths <- c(
  "results/param_baseline", # Model (1)
  "results/param_no_C",     # Model (2)
  "results/param_no_S",     # Model (3)
  "results/param_no_SC",    # Model (4)
  "results/param_no_z",     # Model (5)
  "results/param_no_Cz",    # Model (6)
  "results/param_no_Sz",    # Model (7)
  "results/param_no_SCz"    # Model (8)
)
paths <- paste(paths,".Rdat",sep="")


all.logliks <- NULL
all.Sharpe <- NULL
all.numeric.params <- NULL

for(i.model in 1:length(paths)){
  
  # Load model:
  load(file = paths[[i.model]])
  
  # Make model and compute loglik:
  Model <- THETA.2.MODEL(THETA,kappa,weights.spds)
  res.filter <- run.Filter(Model,DATA,
                           indic.cds=indic.cds, # 1 to use CDS formula (RMV otherwise)
                           indic.smoother = 1, # 1 to compute Risk Premiums
                           indic_max.avg.annual.SharpeRatio = indic_max.avg.annual.SharpeRatio
  )
  
  loglik.i <- sum(res.filter$loglik.vector)
  print(loglik.i)
  
  all.logliks <- c(all.logliks,loglik.i)
  all.Sharpe  <- c(all.Sharpe,mean(res.filter$max.Sharpe.t))
  
  all.param.with.names <- g.complete.new(THETA,kappa,weights.spds,ctry.names)
  
  all.numeric.params <- cbind(all.numeric.params,all.param.with.names$vec.param)
  
}

latex.names.of.params <- all.param.with.names$names.param



# =========================================
# Make Latex Table:

matrix.param <- matrix(c(9:(7+n.e),
                         (8+n.e):(6+2*n.e),
                         (7+2*n.e):(5+3*n.e),
                         7,4,8,
                         6,5,
                         1,2,3,
                         7+3*n.e,8+3*n.e,9+3*n.e,10+3*n.e,
                         6+3*n.e,
                         11+3*n.e,
                         12+3*n.e),ncol=1)

indicate.where.hlines <- c(5,15,17,20,23,28)

indicate.not.estimated <- matrix(0,dim(matrix.param)[1],length(paths))

indicate.not.estimated[6:15,c(2,4,6,8)] <- 1 # no contagion
#indicate.not.estimated[18,1:4] <- 1 # no common factor
indicate.not.estimated[c(16:17,19,24),5:8] <- 1 # no common factor
indicate.not.estimated[27,c(3:4,7:8)] <- 1 # no S

matrix.multipl.factor <- 0*matrix.param

matrix.multipl.factor[1:2] <- 5
matrix.multipl.factor[3:4] <- 4
matrix.multipl.factor[5]   <- 2
matrix.multipl.factor[16]  <- 2
matrix.multipl.factor[17]  <- 3
matrix.multipl.factor[18]  <- 1
matrix.multipl.factor[19]  <- 2
matrix.multipl.factor[20]  <- 2
matrix.multipl.factor[21]  <- 2
matrix.multipl.factor[22]  <- 5
matrix.multipl.factor[23]  <- -5
matrix.multipl.factor[24]  <- 3
matrix.multipl.factor[25]  <- 2
matrix.multipl.factor[26]  <- -2

round.nb <- 3

latex.table <- NULL

aux.string <- "Model&"
for(i in 1:length(paths)){
  aux.string <- paste(aux.string,"&(",toString(i),")",sep="")
}

aux.string <- paste(aux.string,"\\\\",sep="")
latex.table <- rbind(latex.table,aux.string,
                     "\\hline")

count <- 0
for(i in 1:dim(matrix.param)[1]){
  
  if(sum(count %in% indicate.where.hlines)>0){
    latex.table <- rbind(latex.table,
                         "\\hline")
    
  }
  count <- count+1
  
  indic.param <- matrix.param[i]
  
  aux.string <- paste(
    latex.names.of.params[indic.param],
    ifelse(matrix.multipl.factor[i]==0,"&",
           paste("& $\\times 10^{",toString(matrix.multipl.factor[i]),"}$",sep="")),
    sep="")
  
  for(j in 1:length(paths)){
    
    aux.string <- paste(aux.string,"&",
                        ifelse(indicate.not.estimated[i,j]==1,"--",
                               round.fixed.length(all.numeric.params[indic.param,j]*10^matrix.multipl.factor[i],
                                                  round.nb)),
                        sep="")
  }
  aux.string <- paste(aux.string,"\\\\",sep="")
  
  latex.table <- rbind(latex.table,aux.string)
}


latex.table <- rbind(latex.table,
                     "\\hline")

string.logliks <- "log-lik.&"
string.Sharpe <- "Sharpe&"
for(i in 1:length(paths)){
  string.logliks <- paste(string.logliks,"&",round.fixed.length(all.logliks[i],0),sep="")
  string.Sharpe  <- paste(string.Sharpe, "&",round.fixed.length(all.Sharpe[i],2),sep="")
}
string.logliks <- paste(string.logliks,"\\\\",sep="")
string.Sharpe  <- paste(string.Sharpe, "\\\\",sep="")

latex.table <- rbind(latex.table,
                     #"\\hline",
                     string.Sharpe,
                     "\\hline",
                     string.logliks,
                     "\\hline")

latex.file <- paste("results/table_param_multiple_models.txt",sep="")
write(latex.table, file = latex.file)








